@@ -56,6 +56,7 @@ source_modes = [
 
 RESULT_NO_CONNECTION = -2
 RESULT_LOCKED = -3
+RESULT_ABORTED = -4
 
 def get_method(url):
   pos = url.find(":")
@@ -76,6 +77,7 @@ class SyncRunner(object):
     self.sync_watches = None
     self.connection = None
     self.conn_status = None
+    self.aborted = None
     self.poll_cb = None
     self.tick_cb = None
     # determine connection method
@@ -142,6 +144,8 @@ class SyncRunner(object):
     os.close(self.lock_fd)
   def connected(self, connection, event):
     self.conn_status = event.get_status()
+    if self.aborted:
+      return
     if self.conn_status == conic.STATUS_CONNECTED:
       # we're connected, see if the current connection is using proxies
       proxy_mode = self.connection.get_proxy_mode()
@@ -177,6 +181,8 @@ class SyncRunner(object):
   def connect_error(self):
     if self.connection is None:
       return True
+    if self.aborted and not self.state:
+      return True
     if self.conn_status is None or self.conn_status == conic.STATUS_CONNECTED:
       return False
     return True
@@ -204,6 +210,7 @@ class SyncRunner(object):
       return None
     return self.state.progress()
   def abort(self):
+    self.aborted = True
     if self.state is None:
       return
     self.state.abort()
@@ -216,11 +223,16 @@ class SyncRunner(object):
       if self.connection is None:
         status["result"] = RESULT_LOCKED
       else:
-        status["result"] = RESULT_NO_CONNECTION
+        if self.aborted:
+          status["result"] = RESULT_ABORTED
+        else:
+          status["result"] = RESULT_NO_CONNECTION
         self.release_lock()
       return status
     self.deactivate_watches()
     status = self.sync.synchronize_status(self.state)
+    if self.aborted and status["result"]:
+      status["result"] = RESULT_ABORTED
     self.connection.disconnect()
     self.state = None
     self.release_lock()
@@ -847,6 +859,8 @@ class SyncGUI(object):
       banner = hildon.hildon_banner_show_information(self.server_window, "", "Synchronization successful")
     elif status["result"] == RESULT_LOCKED:
       banner = hildon.hildon_banner_show_information(self.server_window, "", "Synchronization already in progress")
+    elif status["result"] == RESULT_ABORTED:
+      banner = hildon.hildon_banner_show_information(self.server_window, "", "Synchronization aborted by user")
     elif status["result"] == RESULT_NO_CONNECTION:
       note = hildon.hildon_note_new_information(self.server_window, "Synchronization failed: no network connection")
       response = gtk.Dialog.run(note)
@@ -927,6 +941,8 @@ class SyncCLI(object):
     elif status["result"] == RESULT_LOCKED:
       # if another synchronization is in progress anyway, this is probably ok to ignore
       pass
+    elif status["result"] == RESULT_ABORTED:
+      self.osso_note.system_note_infoprint("Synchronization aborted by user")
     elif status["result"] == RESULT_NO_CONNECTION:
       self.osso_note.system_note_infoprint("Synchronization failed: No connection")
     else:
